/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: gui_main.c,v 1.15 2000/11/18 02:12:08 guenter Exp $
 *
 * XINE-PANEL 
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>
#include <X11/keysym.h>
#include <Imlib.h>
#include "gui_main.h"
#include "gui_list.h"
#include "gui_button.h"
#include "gui_label.h"
#include "gui_dnd.h"
#include "gui_slider.h"
#include "../xine.h"
#include "../video_out/video_out.h"
#include "../configfile.h"

/*
 * global variables
 */
static Window          gui_panel_win;
static gui_move_t      gui_move; 
static widget_list_t  *gui_widget_list;
static ImlibImage     *gui_bg_image;                 /* background image */
extern ImlibData      *gImlib_data;
extern Display        *gDisplay;
extern vo_driver_t    *gVideoDriver;
extern Window          gVideoWin;

static char            gui_filename[1024];
static widget_t       *gui_title_label;
static char            gui_runtime[20];
static widget_t       *gui_runtime_label;
static int             gui_panel_visible;
static int             gui_running;

/*
 * simple gui playlist
 */
static char           *gui_playlist[100];
static int             gui_playlist_num;
static int             gui_playlist_cur;
static int             gui_ignore_status=0;

extern int nVideoHours ;
extern int nVideoMins ;
extern int nVideoSecs ;

/* 
 * SKINDIR
 */

char *gui_get_skindir(const char file[])
{
  static char tmp[256];
  char *skin;

  skin = config_file_lookup_str ("skin", "default");

  sprintf(tmp, "%s/%s/%s", XINE_SKINDIR, skin, file);

  return tmp;
}

/*
 * Callback-functions for widget-button click
 */

void gui_exit (void) 
{
  printf("xine-panel: EXIT\n");
  xine_exit(); 
  gui_running = 0;
}

void gui_play (void)
{
  /* TODO: find slider in widget_list. now its created as the last widget .. so its easy to find it ;) */

  /*
    FIXME
  widget_t *widget = (widget_t*) gui_list_last_content(gui_widget_list->l);
  
  if (widget->widget_type == WIDGET_TYPE_SLIDER) {
    slider_make_step(widget);
  }
  */

  printf("xine-panel: PLAY\n");
  
  xine_play ( gui_filename, 0 );
}

void gui_dndcallback (char *filename) {
  strcpy (gui_filename, filename);
  set_label (gui_title_label, gui_filename);
  paint_widget_list (gui_widget_list);

  gui_play (); 
}

void gui_stop (void)
{
  gui_ignore_status = 1;
  xine_stop ();
  vo_set_logo_mode (1); 
  gui_ignore_status = 0;
}

void gui_pause (void)
{
  xine_pause ();
}

void gui_toggle_panel_visibility () {

  if (gui_panel_visible) {

    gui_panel_visible = 0;
    XUnmapWindow (gDisplay, gui_panel_win);

  } else {

    gui_panel_visible = 1;
    XMapRaised(gDisplay, gui_panel_win); 
  }

  config_file_set_int ("open_panel", gui_panel_visible);
}

void gui_nothing (void)
{
	// Just do nothing ...
}

/*
 * Function for Loading Images, using ImLib
 */
 
gui_image_t *gui_load_image(char *image)
{
  ImlibImage *img;
  gui_image_t *i;

  i = (gui_image_t*)malloc(sizeof(gui_image_t));

  if( !( img = Imlib_load_image(gImlib_data, image) ) ) {
	  fprintf(stderr, "xine-panel: couldn't find image %s\n", image);
    	  exit(-1);
  }

  Imlib_render (gImlib_data, img, img->rgb_width, img->rgb_height);
  
  i->image  = Imlib_move_image(gImlib_data, img);
  i->width  = img->rgb_width;
  i->height = img->rgb_height;

  return i;
} 

#define MWM_HINTS_DECORATIONS   (1L << 1)
#define PROP_MWM_HINTS_ELEMENTS 5
typedef struct _mwmhints {
  uint32_t flags;
  uint32_t functions;
  uint32_t decorations;
  int32_t  input_mode;
  uint32_t status;
} MWMHints;

void gui_open_panel () {
  GC                      gc;
  XSizeHints              hint;
  XSetWindowAttributes    attr;
  int                     screen;
  char                    title[] = {"Xine Panel"}; /* window-title     */
  input_plugin_t         *ip;
  int                     num_plugins, i;
  Atom                    prop;
  MWMHints                mwmhints;
  XClassHint             *xclasshint;


  if (gui_panel_win)
    return ; /* panel already open  - FIXME: bring to foreground */

  /*
   * load bg image before opening window, so we can determine it's size
   */

  if (!(gui_bg_image = Imlib_load_image(gImlib_data, gui_get_skindir("bg.png")))) {
    fprintf(stderr, "xine-panel: couldn't find image %s\n", "bg.png");
    exit(-1);
  }

  screen = DefaultScreen(gDisplay);
  hint.x = config_file_lookup_int ("x_panel", 200);
  hint.y = config_file_lookup_int ("y_panel", 100);
  hint.width = gui_bg_image->rgb_width;
  hint.height = gui_bg_image->rgb_height;
  hint.flags = PPosition | PSize;
  
  attr.override_redirect = True;
  gui_panel_win = XCreateWindow (gDisplay, DefaultRootWindow(gDisplay), 
				 hint.x, hint.y, hint.width, hint.height, 0, 
				 CopyFromParent, CopyFromParent, 
				 CopyFromParent,
				 0, &attr);
  
  XSetStandardProperties(gDisplay, gui_panel_win, title, title,
			 None, NULL, 0, &hint);
  /*
   * wm, no border please
   */

  prop = XInternAtom(gDisplay, "_MOTIF_WM_HINTS", True);
  mwmhints.flags = MWM_HINTS_DECORATIONS;
  mwmhints.decorations = 0;

  XChangeProperty(gDisplay, gui_panel_win, prop, prop, 32,
                  PropModeReplace, (unsigned char *) &mwmhints,
                  PROP_MWM_HINTS_ELEMENTS);
  
  /* set xclass */

  if((xclasshint = XAllocClassHint()) != NULL) {
    xclasshint->res_name = "Xine Panel";
    xclasshint->res_class = "Xine";
    XSetClassHint(gDisplay, gui_panel_win, xclasshint);
  }

  gc = XCreateGC(gDisplay, gui_panel_win, 0, 0);

  XSelectInput(gDisplay, gui_panel_win,
	       ButtonPressMask | ButtonReleaseMask | PointerMotionMask 
	       | KeyPressMask | ExposureMask | StructureNotifyMask);
  
  Imlib_apply_image(gImlib_data, gui_bg_image, gui_panel_win);
  XSync(gDisplay, False); 

  gui_make_window_dnd_aware (gui_panel_win);

  /*
   * Widget-list
   */
  gui_widget_list = (widget_list_t *) malloc (sizeof (widget_list_t));
  gui_widget_list->l = gui_list_new ();
  gui_widget_list->focusedWidget = NULL;
  gui_widget_list->pressedWidget = NULL;
  gui_widget_list->win           = gui_panel_win;
  gui_widget_list->gc            = gc;
 
  //STOP-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (50, 70, gui_stop, "stop.png"));

  //PLAY-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (90, 70, gui_play,"play.png"));

  // PAUSE-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (130, 70, gui_pause,"pause.png"));

  // EXIT-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (180, 70, gui_exit,"exit.png"));

  // Close-BUTTON
  gui_list_append_content (gui_widget_list->l, 
			   create_button (385, 7, gui_toggle_panel_visibility,
					  "close.png"));

  /* LABEL TITLE */
  gui_title_label = create_label (20,18, 60, gui_filename, 
				  "letters.png");

  gui_list_append_content (gui_widget_list->l, gui_title_label);

  /* runtime label */
  gui_runtime_label = create_label (20, 38, 8, gui_runtime, 
				    "letters.png");

  gui_list_append_content (gui_widget_list->l, gui_runtime_label);


  /* SLIDER */
  /* FIXME
  gui_list_append_content (gui_widget_list->l,
                  create_slider (50, 50, 300, 25, gui_nothing, 1000, 7, 
			  "slider_bg.png",
			  "slider_paddle.png"));
  */

  ip = xine_get_input_plugin_list (&num_plugins);

  printf ("%d input plugins ...\n",num_plugins);

  for (i=0; i<num_plugins; i++) {
    printf ("plugin #%d : id=%s\n",i,ip->get_identifier());
    ip++;
  }

}

void gui_set_current_position (int pos) {
  gui_ignore_status = 1;
  xine_stop ();
  xine_play ( gui_filename, pos);
  gui_ignore_status = 0;
}

void gui_handle_event (XEvent *event) {
  XKeyEvent      mykeyevent;
  XExposeEvent  *myexposeevent;
  KeySym         mykey;
  static XEvent *old_event;
  char kbuf[256];
  int len;

  switch(event->type) {
  case Expose: {
    myexposeevent = (XExposeEvent *) event;

    if (myexposeevent->window == gVideoWin)
      gVideoDriver->handle_event (event);
    else
      paint_widget_list (gui_widget_list);
    
  }
  break;
    
  case MotionNotify:

    motion_notify_widget_list (gui_widget_list, event->xbutton.x, event->xbutton.y);
    /* if window-moving is enabled move the window */
    old_event = event;
    if (gui_move.enabled) {
      int x,y;
      x = (event->xmotion.x_root) + ( event->xmotion.x_root - old_event->xmotion.x_root) - gui_move.offset_x;
      y = (event->xmotion.y_root) + ( event->xmotion.y_root - old_event->xmotion.y_root) - gui_move.offset_y;

      XLockDisplay (gDisplay);
      XMoveWindow(gDisplay, gui_panel_win, x, y);
      XUnlockDisplay (gDisplay);

      config_file_set_int ("x_panel",x);
      config_file_set_int ("y_panel",y);

    }
    break;

  case MappingNotify:
    XLockDisplay (gDisplay);
    XRefreshKeyboardMapping((XMappingEvent *) event);
    XUnlockDisplay (gDisplay);
    break;

  case DestroyNotify:
    xine_exit ();
    gui_running = 0;
    break;

  case ButtonPress: {
    XButtonEvent *bevent = (XButtonEvent *) event;

    /* printf ("button: %d\n",bevent->button); */

    if ((bevent->button==3) && (bevent->window == gVideoWin)) {
      gui_toggle_panel_visibility ();
      break;
    }
    
    /* if no widget is hit enable moving the window */
    gui_move.enabled = !click_notify_widget_list (gui_widget_list, 
						  event->xbutton.x, 
						  event->xbutton.y, 0);
    if (gui_move.enabled) {
      gui_move.offset_x = event->xbutton.x;
      gui_move.offset_y = event->xbutton.y;
    }
  }
  break;

  case ButtonRelease:
    click_notify_widget_list (gui_widget_list, event->xbutton.x, 
			      event->xbutton.y,1);
    gui_move.enabled = 0; /* disable moving the window       */  
    break;

  case KeyPress:
    mykeyevent = event->xkey;
      
    XLockDisplay (gDisplay);
    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
    XUnlockDisplay (gDisplay);

    switch (mykey) {
    case XK_P:
    case XK_p:
      gui_pause (); 
      break;
    case XK_f:
    case XK_F:
      vo_set_fullscreen (!vo_is_fullscreen ());
      if (gui_panel_visible) 
	XRaiseWindow (gDisplay, gui_panel_win);

      break;
    case XK_q:
    case XK_Q:
      gui_exit();
      break;
    case XK_Return:
      gui_play();
      break;
    case XK_space:
      gui_pause();
      break;

    case XK_Next:
      xine_stop ();
      break;

    case XK_Prior:
      gui_ignore_status = 1;
      xine_stop ();
      gui_playlist_cur--;
      if ((gui_playlist_cur>=0) && (gui_playlist_cur < gui_playlist_num)) {
	strcpy(gui_filename, gui_playlist [gui_playlist_cur]);
	set_label (gui_title_label, gui_filename);
	paint_label (gui_title_label, gui_panel_win, gui_widget_list->gc);
	xine_play ( gui_filename, 0 );
      } else {
	vo_set_logo_mode (1); 
	gui_playlist_cur = 0;
      }
      gui_ignore_status = 0;
      break;
      
    case XK_1:
      gui_set_current_position (6553);
      break;
    case XK_2:
      gui_set_current_position (13107);
      break;
    case XK_3:
      gui_set_current_position (19660);
      break;
    case XK_4:
      gui_set_current_position (26214);
      break;
    case XK_5:
      gui_set_current_position (32767);
      break;
    case XK_6:
      gui_set_current_position (39321);
      break;
    case XK_7:
      gui_set_current_position (45874);
      break;
    case XK_8:
      gui_set_current_position (52428);
      break;
    case XK_9:
      gui_set_current_position (58981);
      break;
    case XK_0:
      gui_set_current_position (0);
      break;
      
    }

    break;

  case ConfigureNotify:
    /*  background */
    XLockDisplay (gDisplay);
    Imlib_apply_image(gImlib_data, gui_bg_image, gui_panel_win);
    XUnlockDisplay (gDisplay);
    paint_widget_list (gui_widget_list);
    XLockDisplay (gDisplay);
    XSync(gDisplay, False);
    XUnlockDisplay (gDisplay);
    break;

  case ClientMessage:
    gui_dnd_process_client_message (event);
    break;
    /*
      default:
      printf("Got event: %i\n", event->type);
    */
  }

}

void gui_status_callback (int nStatus) {

  if (gui_ignore_status) 
    return;

  /* printf ("gui status callback : %d\n", nStatus); */

  if (gui_panel_visible) {
    sprintf (gui_runtime, "%02d:%02d:%02d", nVideoHours, nVideoMins, nVideoSecs);

    set_label (gui_runtime_label, gui_runtime);

    paint_label (gui_runtime_label, gui_panel_win, gui_widget_list->gc);
  }

  if (nStatus == XINE_STOP) {
    gui_playlist_cur++;
    if (gui_playlist_cur < gui_playlist_num) {
      strcpy(gui_filename, gui_playlist [gui_playlist_cur]);
      set_label (gui_title_label, gui_filename);
      paint_label (gui_title_label, gui_panel_win, gui_widget_list->gc);
      xine_play ( gui_filename, 0 );
    } else {
	vo_set_logo_mode (1); 
    }
  }
}

void gui_start (int nfiles, char *filenames[]) {

  XEvent  myevent;
  int     i;

  /*
   * moving the window
   */

  gui_move.enabled  = 0;
  gui_move.offset_x = 0;
  gui_move.offset_y = 0; 

  for (i=0; i<nfiles; i++)
    gui_playlist[i]     = filenames[i];
  gui_playlist_num = nfiles; 
  gui_playlist_cur = 0;

  if (nfiles)
    strcpy(gui_filename, gui_playlist [gui_playlist_cur]);
  else 
    sprintf(gui_filename, "DROP A FILE ON XINE");

  sprintf (gui_runtime, "%02d:%02d:%02d", nVideoHours, nVideoMins, nVideoSecs);

  XLockDisplay (gDisplay);
  
  /*
   * setup panel
   */

  gui_open_panel ();

  if (config_file_lookup_int ("open_panel", 1)) {
    XMapRaised(gDisplay, gui_panel_win); 
    gui_panel_visible = 1;
  } else
    gui_panel_visible = 0;

  XUnlockDisplay (gDisplay);

  /*
   * event loop
   */

  gui_running = 1;

  while (gui_running) {
    XNextEvent (gDisplay, &myevent) ;

    gui_handle_event (&myevent) ;
  }
}

